package org.goya.server.test.dynamic;

import org.testng.annotations.Test;
import org.goya.server.dynamic.dao.MetaModelDAO;
import org.goya.server.dynamic.model.*;
import org.goya.server.test.runtime.EJB3Container;

import java.util.Collection;

public class MetaModel {

    @Test(groups = "unit.metamodel")
    public MetaType createMetaType() {
        MetaType metaType = new MetaType();
        metaType.setName("test");
        metaType.setDescription("This is a type");
        metaType.addAttribute(MetaAttributeType.STRING, "foo", "An attribute");

        // Check state
        assert metaType.getAttributes().size() == 1;
        assert metaType.getAttributes().get("foo").getMetaType().equals(metaType);
        assert metaType.getAttributes().get("foo").getAttributeType().equals(MetaAttributeType.STRING);

        return metaType;
    }

    @Test(groups = "unit.metamodel")
    public Item createItem() {
        MetaType metaType = createMetaType();
        Item item = new Item(metaType);
        item.addAttributeValue("foo", "bar");

        // Check state
        assert item.getAttributeValues().size() == 1;
        assert item.getAttributeValues().iterator().next().getValue().equals("bar");

        return item;
    }

    @Test(groups = "integration.persistence")
    public void createAndQueryMetadata() {

        Item item = createItem();
        MetaType metaType = item.getMetaType();

        MetaModelDAO dao = (MetaModelDAO) EJB3Container.lookup(MetaModelDAO.class);
        dao.makePersistent(metaType);
        dao.makePersistent(item);

        // Check state
        MetaType ctrlMetaType = dao.findMetaTypeByName("test");
        assert ctrlMetaType.getId().equals(metaType.getId());

        Collection items = dao.findItemsByMetaType(ctrlMetaType);
        assert items.size() == 1;

        Collection itemsWithAttribute = dao.findItemsByAttribute( metaType.getAttributes().get("foo") );
        assert itemsWithAttribute.size() == 1;

        Collection itemsWithValue = dao.findItemsByAttributeValue(MetaAttributeType.STRING, "bar");
        assert itemsWithValue.size() == 1;
    }

}
